@@ -3,17 +3,21 @@ package Module::Provision::Base;
 use namespace::autoclean;
 
 use Moo;
-use Class::Usul::Constants;
-use Class::Usul::Functions  qw( app_prefix class2appdir classdir io throw );
+use Class::Usul::Constants qw( EXCEPTION_CLASS NUL SPC TRUE );
+use Class::Usul::Functions qw( app_prefix class2appdir classdir
+                               distname first_char io is_arrayref throw );
 use Class::Usul::Options;
-use Class::Usul::Time       qw( time2str );
-use English                 qw( -no_match_vars );
-use File::DataClass::Types  qw( ArrayRef Directory HashRef NonEmptySimpleStr
-                                Object OctalNum Path PositiveInt SimpleStr );
+use Class::Usul::Time      qw( time2str );
+use CPAN::Meta;
+use English                qw( -no_match_vars );
+use File::DataClass::Types qw( ArrayRef Directory HashRef NonEmptySimpleStr
+                               Object OctalNum Path PositiveInt
+                               SimpleStr Undef );
 use Module::Metadata;
 use Perl::Version;
-use Type::Utils             qw( enum );
-use Unexpected::Functions   qw( Unspecified );
+use Try::Tiny;
+use Type::Utils            qw( enum );
+use Unexpected::Functions  qw( Unspecified );
 
 extends q(Class::Usul::Programs);
 
@@ -26,9 +30,9 @@ has '+config_class' => default => sub { 'Module::Provision::Config' };
 
 # Object attributes (public)
 #   Visible to the command line
-option 'base'       => is => 'lazy', isa => Path, format => 's',
+option 'base'       => is => 'lazy', isa => Path, coerce => TRUE,
    documentation    => 'Directory containing new projects',
-   coerce           => Path->coercion, default => sub { $_[ 0 ]->config->base };
+   builder          => sub { $_[ 0 ]->config->base }, format => 's';
 
 option 'branch'     => is => 'lazy', isa => SimpleStr, format => 's',
    documentation    => 'The name of the initial branch to create', short => 'b';
@@ -38,85 +42,155 @@ option 'builder'    => is => 'lazy', isa => $BUILDER, format => 's',
 
 option 'license'    => is => 'ro',   isa => NonEmptySimpleStr, format => 's',
    documentation    => 'License used for the project',
-   default          => sub { $_[ 0 ]->config->license };
+   builder          => sub { $_[ 0 ]->config->license };
 
-option 'perms'      => is => 'ro',   isa => OctalNum, format => 'i',
+option 'perms'      => is => 'ro',   isa => OctalNum, coerce => TRUE,
    documentation    => 'Default permission for file / directory creation',
-   coerce           => OctalNum->coercion, default => '640';
+   default          => '640', format => 'i';
+
+option 'plugins'    => is => 'ro',   isa => ArrayRef[NonEmptySimpleStr],
+   documentation    => 'Name of optional plugins to load, comma seprated list',
+   builder          => sub { [] }, format => 's', short => 'M',
+   coerce           => sub { (is_arrayref $_[ 0 ])
+                                ? $_[ 0 ] : [ split m{ , }mx, $_[ 0 ] ] };
 
 option 'project'    => is => 'lazy', isa => NonEmptySimpleStr, format => 's',
    documentation    => 'Package name of the new projects main module';
 
 option 'repository' => is => 'ro',   isa => NonEmptySimpleStr, format => 's',
    documentation    => 'Directory containing the SVN repository',
-   default          => sub { $_[ 0 ]->config->repository };
+   builder          => sub { $_[ 0 ]->config->repository };
 
 option 'vcs'        => is => 'lazy', isa => $VCS, format => 's',
    documentation    => 'Which VCS to use: git, none, or svn';
 
 #   Ingnored by the command line
-has 'appbase'         => is => 'lazy', isa => Path, coerce => Path->coercion;
+has 'appbase'         => is => 'lazy', isa => Path, coerce => TRUE;
 
-has 'appldir'         => is => 'lazy', isa => Path, coerce => Path->coercion;
+has 'appldir'         => is => 'lazy', isa => Path, coerce => TRUE;
 
-has 'branch_file'     => is => 'lazy', isa => Path, coerce => Path->coercion,
-   default            => sub { [ $_[ 0 ]->appbase, '.branch' ] };
+has 'branch_file'     => is => 'lazy', isa => Path, coerce => TRUE,
+   builder            => sub { [ $_[ 0 ]->appbase, '.branch' ] };
 
-has 'binsdir'         => is => 'lazy', isa => Path, coerce => Path->coercion,
-   default            => sub { [ $_[ 0 ]->appldir, 'bin' ] };
+has 'binsdir'         => is => 'lazy', isa => Path, coerce => TRUE,
+   builder            => sub { [ $_[ 0 ]->appldir, 'bin' ] };
 
 has 'default_branch'  => is => 'lazy', isa => SimpleStr;
 
-has 'dist_module'     => is => 'lazy', isa => Path, coerce => Path->coercion,
-   default            => sub { [ $_[ 0 ]->homedir.'.pm' ] };
+has 'dist_module'     => is => 'lazy', isa => Path, coerce => TRUE,
+   builder            => sub { [ $_[ 0 ]->homedir.'.pm' ] };
 
 has 'dist_version'    => is => 'lazy', isa => Object;
 
 has 'distname'        => is => 'lazy', isa => NonEmptySimpleStr,
-   default            => sub { __distname( $_[ 0 ]->project ) };
+   builder            => sub { distname $_[ 0 ]->project };
 
 has 'exec_perms'      => is => 'lazy', isa => PositiveInt;
 
-has 'homedir'         => is => 'lazy', isa => Path, coerce => Path->coercion;
+has 'homedir'         => is => 'lazy', isa => Path, coerce => TRUE;
 
-has 'incdir'          => is => 'lazy', isa => Path, coerce => Path->coercion,
-   default            => sub { [ $_[ 0 ]->appldir, 'inc' ] };
+has 'incdir'          => is => 'lazy', isa => Path, coerce => TRUE,
+   builder            => sub { [ $_[ 0 ]->appldir, 'inc' ] };
 
 has 'initial_wd'      => is => 'ro',   isa => Directory,
-   default            => sub { io()->cwd };
+   builder            => sub { io()->cwd };
 
-has 'libdir'          => is => 'lazy', isa => Path, coerce => Path->coercion,
-   default            => sub { [ $_[ 0 ]->appldir, 'lib' ] };
+has 'libdir'          => is => 'lazy', isa => Path, coerce => TRUE,
+   builder            => sub { [ $_[ 0 ]->appldir, 'lib' ] };
+
+has 'license_keys'    => is => 'lazy', isa => HashRef;
 
 has 'manifest_paths'  => is => 'lazy', isa => ArrayRef, init_arg => undef;
 
 has 'module_abstract' => is => 'lazy', isa => NonEmptySimpleStr;
 
+has 'module_metadata' => is => 'lazy', isa => Object | Undef, builder => sub {
+   Module::Metadata->new_from_file
+      ( $_[ 0 ]->dist_module->abs2rel( $_[ 0 ]->appldir ), collect_pod => 1 ) };
+
 has 'project_file'    => is => 'lazy', isa => NonEmptySimpleStr;
 
 has 'stash'           => is => 'lazy', isa => HashRef;
 
-has 'testdir'         => is => 'lazy', isa => Path, coerce => Path->coercion,
-   default            => sub { [ $_[ 0 ]->appldir, 't' ] };
+has 'testdir'         => is => 'lazy', isa => Path, coerce => TRUE,
+   builder            => sub { [ $_[ 0 ]->appldir, 't' ] };
 
-# Object attributes (private)
-has '_license_keys'   => is => 'lazy', isa => HashRef;
+# Private functions
+my $_builders = sub {
+   return (sort keys %BUILDERS);
+};
 
-# Public methods
-sub chdir {
-   my ($self, $dir) = @_;
+my $_get_module_from = sub { # Return main module name from project file
+   return
+      (map    { s{ [-] }{::}gmx; $_ }
+       map    { m{ \A [q\'\"] }mx ? eval $_ : $_ }
+       map    { m{ \A \s* (?:module_name|module|name)
+                      \s+ [=]?[>]? \s* ([^,;]+) [,;]? }imx }
+       grep   { m{ \A \s*   (module|name) }imx }
+       split m{ [\n] }mx, $_[ 0 ])[ 0 ];
+};
 
-         $dir or throw class => Unspecified, args => [ 'directory' ];
-   chdir $dir or throw error => 'Directory [_1] cannot chdir: [_2]',
-                       args  => [ $dir, $OS_ERROR ];
-   return $dir;
+my $_parse_manifest_line = sub { # Robbed from ExtUtils::Manifest
+   my $line = shift; my ($file, $comment);
+
+   # May contain spaces if enclosed in '' (in which case, \\ and \' are escapes)
+   if (($file, $comment) = $line =~ m{ \A \' (\\[\\\']|.+)+ \' \s* (.*) }mx) {
+      $file =~ s{ \\ ([\\\']) }{$1}gmx;
+   }
+   else {
+       ($file, $comment) = $line =~ m{ \A (\S+) \s* (.*) }mx;
+   }
+
+   return [ $file, $comment ];
+};
+
+my $_get_project_file = sub {
+   my $dir = shift; my $prev;
+
+   while (not $prev or $prev ne $dir) { # Search for dist.ini first
+      for my $file (grep { $_->exists }
+                    map  { $dir->catfile( $BUILDERS{ $_ } ) } $_builders->()) {
+         return $file
+      }
+
+      $prev = $dir; $dir = $dir->parent;
+   }
+
+   return;
+};
+
+# Construction
+sub BUILD {
+   my $self = shift;
+
+   for my $plugin (@{ $self->plugins }) {
+      if (first_char $plugin eq '+') { $plugin = substr $plugin, 1 }
+      else { $plugin = "Module::Provision::TraitFor::${plugin}" }
+
+      try   { Role::Tiny->apply_roles_to_object( $self, $plugin ) }
+      catch {
+         $_ =~ m{ \ACan\'t \s+ locate }mx or throw $_;
+         throw 'Module [_1] not found in @INC', [ $plugin ];
+      };
+   }
+
+   return;
 }
 
-# Private methods
-sub _build_appbase {
+sub _build_appbase { # Base + distname
    my $self = shift; my $base = $self->base->absolute( $self->initial_wd );
 
-   return $base->catdir( $self->distname );
+   my $appbase = $base->catdir( $self->distname );
+
+   $appbase->exists and return $appbase;
+
+   # This is so you can rename the dist directory
+   my $file         = $_get_project_file->( $self->initial_wd );
+   my $grand_parent = $file && $file->parent && $file->parent->parent;
+
+   $grand_parent and $grand_parent->exists
+      and $grand_parent !~ m{ \.build \z }mx and return $grand_parent;
+   return $appbase;
 }
 
 sub _build_appldir {
@@ -148,7 +222,7 @@ sub _build_builder {
    my $self = shift; my $appldir = $self->appldir;
 
    for (grep { $_->[ 1 ]->exists }
-        map  { [ $_, $appldir->catfile( $BUILDERS{ $_ } ) ] } __builders()) {
+        map  { [ $_, $appldir->catfile( $BUILDERS{ $_ } ) ] } $_builders->()) {
       return $_->[ 0 ];
    }
 
@@ -160,11 +234,9 @@ sub _build_default_branch {
 }
 
 sub _build_dist_version {
-   my $self   = shift;
-   my $module = $self->dist_module->abs2rel( $self->appldir );
-   my $info   = Module::Metadata->new_from_file( $module );
+   my $self = shift; my $meta = $self->module_metadata;
 
-   return Perl::Version->new( $info ? $info->version : '0.1.1' );
+   return Perl::Version->new( $meta ? $meta->version : '0.1.1' );
 }
 
 sub _build_exec_perms {
@@ -175,7 +247,7 @@ sub _build_homedir {
    return [ $_[ 0 ]->libdir, classdir $_[ 0 ]->project ];
 }
 
-sub _build__license_keys {
+sub _build_license_keys {
    return {
       perl       => 'Perl_5',
       perl_5     => 'Perl_5',
@@ -193,31 +265,28 @@ sub _build_manifest_paths {
    my $self = shift;
 
    return [ grep { $_->exists }
-            map  { io( __parse_manifest_line( $_ )->[ 0 ] ) }
+            map  { io( $_parse_manifest_line->( $_ )->[ 0 ] ) }
             grep { not m{ \A \s* [\#] }mx }
             $self->appldir->catfile( 'MANIFEST' )->chomp->getlines ];
 }
 
 sub _build_module_abstract {
-   return $_[ 0 ]->loc( $_[ 0 ]->config->module_abstract );
+   my $self = shift; my $meta = $self->module_metadata; my $abstract = NUL;
+
+   $meta and ($abstract = $meta->pod( 'Name' ) // NUL)
+      =~ s{ \A [^\-]+ \s* [\-] \s* }{}mx; chomp $abstract;
+
+   return $self->loc( $abstract || $self->config->module_abstract );
 }
 
 sub _build_project {
-   my $self = shift; my $dir = $self->initial_wd; my ($prev, $module);
+   my $self   = shift;
+   my $file   = $_get_project_file->( $self->initial_wd )
+      or throw 'Path [_1] contains no project file', [ $self->initial_wd ];
+   my $module = $_get_module_from->( $file->all )
+      or throw 'File [_1] contains no module name', [ $file ];
 
-   while (not $prev or $prev ne $dir) { # Search for dist.ini first
-      for my $file (grep { $_->exists }
-                    map  { $dir->catfile( $BUILDERS{ $_ } ) } __builders()) {
-         $module = __get_module_from( $file->all ) and return $module;
-         throw error => 'File [_1] contains no module name', args => [ $file ];
-      }
-
-      $prev = $dir; $dir = $dir->parent;
-   }
-
-   throw error => 'Path [_1] contains no project files',
-         args  => [ $self->initial_wd ];
-   return; # Never reached
+   return $module;
 }
 
 sub _build_project_file {
@@ -249,7 +318,7 @@ sub _build_stash {
             last_name      => lc ((split SPC, $author)[ -1 ]),
             lc_distname    => lc $self->distname,
             license        => $self->license,
-            license_class  => $self->_license_keys->{ $self->license },
+            license_class  => $self->license_keys->{ $self->license },
             module         => $project,
             perl           => $perl_ver,
             prefix         => (split m{ :: }mx, lc $project)[ -1 ],
@@ -262,43 +331,31 @@ sub _build_vcs {
    my $self = shift; my $appbase = $self->appbase;
 
    return $appbase->catdir( '.git'            )->exists ? 'git'
-        : $appbase->catdir( qw( master .git ) )->exists ? 'git'
+        : $appbase->catdir( 'master', '.git'  )->exists ? 'git'
         : $appbase->catdir( '.svn'            )->exists ? 'svn'
-        : $appbase->catdir( qw( trunk  .svn ) )->exists ? 'svn'
+        : $appbase->catdir( 'trunk', '.svn'   )->exists ? 'svn'
         : $appbase->catdir( $self->repository )->exists ? 'svn'
                                                         : 'none';
 }
 
-# Private functions
-sub __builders {
-   return (sort keys %BUILDERS);
+# Public methods
+sub chdir {
+   my ($self, $dir) = @_;
+
+         $dir or throw Unspecified, [ 'directory' ];
+   chdir $dir or throw 'Directory [_1] cannot chdir: [_2]', [ $dir, $OS_ERROR ];
+   return $dir;
 }
 
-sub __distname {
-   (my $y = $_[ 0 ] || NUL) =~ s{ :: }{-}gmx; return $y;
-}
+sub load_meta {
+   my ($self, $dir) = @_;
 
-sub __get_module_from { # Return main module name from contents of project file
-   return
-      (map    { s{ [-] }{::}gmx; $_ }
-       map    { m{ \A [q\'\"] }mx ? eval $_ : $_ }
-       map    { m{ \A \s* (?:module_name|module|name) \s+ [=]?[>]? \s* ([^,;]+) [,;]? }imx }
-       grep   { m{ \A \s*   (module|name) }imx }
-       split m{ [\n] }mx, $_[ 0 ])[ 0 ];
-}
+   $dir or $dir = $self->builder eq 'DZ'
+                ? io( $self->distname.'-'.$self->dist_version ) : undef;
 
-sub __parse_manifest_line { # Robbed from ExtUtils::Manifest
-   my $line = shift; my ($file, $comment);
+   my $path = $dir ? $dir->catfile( 'META.json' ) : io 'META.json';
 
-   # May contain spaces if enclosed in '' (in which case, \\ and \' are escapes)
-   if (($file, $comment) = $line =~ m{ \A \' (\\[\\\']|.+)+ \' \s* (.*) }mx) {
-      $file =~ s{ \\ ([\\\']) }{$1}gmx;
-   }
-   else {
-       ($file, $comment) = $line =~ m{ \A (\S+) \s* (.*) }mx;
-   }
-
-   return [ $file, $comment ];
+   return CPAN::Meta->load_file( "${path}" );
 }
 
 1;
@@ -364,6 +421,10 @@ Permissions used to create files. Defaults to C<644>. Directories and
 programs have the execute bit turned on if the corresponding read bit
 is on
 
+=item C<plugins>
+
+Optional trait to load and apply
+
 =item C<project>
 
 The class name of the new project. Should be the first extra argument on the
@@ -382,6 +443,10 @@ or C<svn>
 
 =head1 Subroutines/Methods
 
+=head2 BUILD
+
+Load and apply optional traits
+
 =head2 chdir
 
    $directory = $self->chdir( $directory );
@@ -389,6 +454,12 @@ or C<svn>
 Changes the current working directory to the one supplied and returns it.
 Throws if the operation was not successful
 
+=head2 load_meta
+
+   $cpan_meta_object = $self->load_meta( $optional_directory );
+
+Loads the F<META.json> file and returns and object
+
 =head1 Diagnostics
 
 None
@@ -429,7 +500,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2013 Peter Flanigan. All rights reserved
+Copyright (c) 2015 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
