@@ -3,47 +3,55 @@ package Module::Provision::Config;
 use namespace::autoclean;
 
 use Moo;
-use Class::Usul::Constants;
-use Class::Usul::Functions  qw( fullname loginid logname untaint_cmdline
-                                untaint_identifier );
-use File::DataClass::Types  qw( ArrayRef HashRef NonEmptySimpleStr
-                                Path SimpleStr );
+use Class::Usul::Constants qw( NUL TRUE );
+use Class::Usul::Functions qw( fullname loginid logname untaint_cmdline
+                               untaint_identifier );
+use File::DataClass::Types qw( ArrayRef HashRef NonEmptySimpleStr
+                               Path SimpleStr );
 
 extends qw(Class::Usul::Config::Programs);
 
 # Object attributes (public)
-has 'author'           => is => 'lazy', isa => NonEmptySimpleStr;
+has 'author'           => is => 'lazy', isa => NonEmptySimpleStr,
+   builder             => sub {
+      my $author =  untaint_cmdline $ENV{AUTHOR} || fullname || logname;
+         $author =~ s{ [\'] }{\'}gmx; return $author };
 
-has 'author_email'     => is => 'lazy', isa => NonEmptySimpleStr;
+has 'author_email'     => is => 'lazy', isa => NonEmptySimpleStr,
+   builder             => sub {
+      my $email =  untaint_cmdline $ENV{EMAIL} || 'dave@example.com';
+         $email =~ s{ [\'] }{\'}gmx; return $email };
 
 has 'author_id'        => is => 'lazy', isa => NonEmptySimpleStr,
-   default             => sub { loginid };
+   builder             => sub { loginid };
 
-has 'base'             => is => 'lazy', isa => Path, coerce => Path->coercion,
-   default             => sub { $_[ 0 ]->my_home };
+has 'base'             => is => 'lazy', isa => Path, coerce => TRUE,
+   builder             => sub { $_[ 0 ]->my_home };
 
 has 'builder'          => is => 'lazy', isa => NonEmptySimpleStr,
    default             => 'MB';
 
 has 'default_branches' => is => 'lazy', isa => HashRef,
-   default             => sub { { git => 'master', svn => 'trunk' } };
+   builder             => sub { { git => 'master', svn => 'trunk' } };
 
-has 'delete_files_uri' => is => 'lazy', isa => NonEmptySimpleStr;
+has 'delete_files_uri' => is => 'lazy', isa => NonEmptySimpleStr,
+   builder             => sub { untaint_cmdline $ENV{CPAN_DELETE_FILES_URI}
+                                || 'https://pause.perl.org/pause/authenquery' };
 
 has 'editor'           => is => 'lazy', isa => NonEmptySimpleStr,
-   default             => sub { untaint_identifier $ENV{EDITOR} || 'emacs' };
+   builder             => sub { untaint_identifier $ENV{EDITOR} || 'emacs' };
 
 has 'home_page'        => is => 'lazy', isa => NonEmptySimpleStr,
    default             => 'http://example.com';
 
 has 'hooks'            => is => 'lazy', isa => ArrayRef[NonEmptySimpleStr],
-   default             => sub { [ 'commit-msg', 'pre-commit' ] };
+   builder             => sub { [ 'commit-msg', 'pre-commit' ] };
 
 has 'license'          => is => 'lazy', isa => NonEmptySimpleStr,
    default             => 'perl';
 
 has 'min_perl_ver'     => is => 'lazy', isa => NonEmptySimpleStr,
-   default             => '5.01';
+   default             => '5.010001';
 
 has 'module_abstract'  => is => 'lazy', isa => NonEmptySimpleStr,
    default             => 'One-line description of the modules purpose';
@@ -51,8 +59,8 @@ has 'module_abstract'  => is => 'lazy', isa => NonEmptySimpleStr,
 has 'repository'       => is => 'lazy', isa => NonEmptySimpleStr,
    default             => 'repository';
 
-has 'seed_file'        => is => 'lazy', isa => Path, coerce => Path->coercion,
-   default             => sub { [ qw( ~ .ssh pause.key ) ] };
+has 'seed_file'        => is => 'lazy', isa => Path, coerce => TRUE,
+   builder             => sub { [ qw( ~ .ssh pause.key ) ] };
 
 has 'signing_key'      => is => 'lazy', isa => SimpleStr,
    default             => NUL;
@@ -65,30 +73,12 @@ has 'template_index'   => is => 'lazy', isa => NonEmptySimpleStr,
 
 has 'test_env_vars'    => is => 'lazy', isa => ArrayRef,
    documentation       => 'Set these environment vars to true when testing',
-   default             => sub {
+   builder             => sub {
       [ qw( AUTHOR_TESTING TEST_MEMORY TEST_SPELLING ) ] };
 
 has 'vcs'              => is => 'lazy', isa => NonEmptySimpleStr,
    default             => 'git';
 
-# Private methods
-sub _build_author {
-   my $author = untaint_cmdline( $ENV{AUTHOR} || fullname || logname );
-
-   $author =~ s{ [\'] }{\'}gmx; return $author;
-}
-
-sub _build_author_email {
-   my $email = untaint_cmdline( $ENV{EMAIL} || 'dave@example.com' );
-
-   $email =~ s{ [\'] }{\'}gmx; return $email;
-}
-
-sub _build_delete_files_uri {
-   return untaint_cmdline $ENV{CPAN_DELETE_FILES_URI}
-       || 'https://pause.perl.org/pause/authenquery';
-}
-
 1;
 
 __END__
@@ -129,6 +119,10 @@ Defines the following attributes;
 
 =item C<builder>
 
+=item C<default_branches>
+
+=item C<delete_files_uri>
+
 =item C<editor>
 
 =item C<home_page>
@@ -137,10 +131,18 @@ Defines the following attributes;
 
 =item C<license>
 
+=item C<min_perl_ver>
+
 =item C<module_abstract>
 
 =item C<repository>
 
+=item C<seed_file>
+
+=item C<signing_key>
+
+=item C<tag_message>
+
 =item C<template_index>
 
 =item C<test_env_vars>
@@ -150,6 +152,9 @@ to; C<AUTHOR_TESTING TEST_MEMORY>, and C<TEST_SPELLING>
 
 =item C<vcs>
 
+A non empty simple string that defaults to C<git>. The default version control
+system
+
 =back
 
 =head1 Subroutines/Methods
@@ -192,7 +197,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2013 Peter Flanigan. All rights reserved
+Copyright (c) 2015 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
